/**
 * Available methods:
 * * - All/Wildcard
 * n - None
 * c - Create
 * r - Read
 * u - Update
 * d - Delete
 * x - Execute
 */
export type Method = '*' | 'n' | 'c' | 'r' | 'u' | 'd' | 'x';
/**
 * Shorthand for creating PathedEvent from a string
 *
 * @example
 * ```ts
 * const event: PathedEvent = PE`users/system:*`;
 * ```
 *
 * @param {TemplateStringsArray} str String that will be parsed into PathedEvent
 * @param {string} args
 * @return {PathEvent} PathedEvent object
 */
export declare function PE(str: TemplateStringsArray, ...args: string[]): PathEvent;
/**
 * Shorthand for creating PathedEvent strings, ensures paths are correct
 *
 * @param {TemplateStringsArray} str
 * @param {string} args
 * @return {string}
 * @constructor
 */
export declare function PES(str: TemplateStringsArray, ...args: any[]): string;
/**
 * A pathed event broken down into its core components for easy processing
 * PathedEvent Structure: `module/path/name:property:method`
 * Example: `users/system:crud` or `storage/some/path/file.txt:r`
 */
export declare class PathEvent {
    /** First directory in path */
    module: string;
    /** Entire path, including the module & name */
    fullPath: string;
    /** Path including the name, excluding the module */
    path: string;
    /** Last sagment of path */
    name: string;
    /** List of methods */
    methods: Method[];
    /** All/Wildcard specified */
    all: boolean;
    /** None specified */
    none: boolean;
    /** Create method specified */
    create: boolean;
    /** Read method specified */
    read: boolean;
    /** Update method specified */
    update: boolean;
    /** Delete method specified */
    delete: boolean;
    constructor(pathedEvent: string | PathEvent);
    /**
     * Combine multiple pathed events into one parsed object. Longest path takes precedent, but all subsequent methods are
     * combined until a "none" is reached
     *
     * @param {string | PathEvent} paths PathedEvents as strings or pre-parsed
     * @return {PathEvent} Final combined permission
     */
    static combine(paths: (string | PathEvent)[]): PathEvent;
    /**
     * Squash 2 sets of paths & return true if any overlap is found
     *
     * @param {string | PathEvent | (string | PathEvent)[]} target Array of PathedEvents as strings or pre-parsed
     * @param has
     * @return {boolean} Whether there is any overlap
     */
    static has(target: string | PathEvent | (string | PathEvent)[], ...has: (string | PathEvent)[]): boolean;
    /**
     * Same as `has` but raises an error if there is no overlap
     *
     * @param {string | string[]} target Array of PathedEvents as strings or pre-parsed
     * @param has
     */
    static hasFatal(target: string | PathEvent | (string | PathEvent)[], ...has: (string | PathEvent)[]): void;
    /**
     * Create pathed event string from its components
     *
     * @param {string | string[]} path Event path
     * @param {Method} methods Event method
     * @return {string} String representation of PathedEvent
     */
    static toString(path: string | string[], methods: Method | Method[]): string;
    /**
     * Create pathed event string from its components
     *
     * @return {string} String representation of PathedEvent
     */
    toString(): string;
}
export type Listener = (event: PathEvent, ...args: any[]) => any;
export type Unsubscribe = () => void;
export interface IPathedEventEmitter {
    emit(event: string, ...args: any[]): void;
    off(listener: Listener): void;
    on(event: string, listener: Listener): Unsubscribe;
    once(event: string, listener?: Listener): Promise<any>;
    relayEvents(emitter: PathedEventEmitter): void;
}
/**
 * Event emitter that uses paths allowing listeners to listen to different combinations of modules, paths & methods
 */
export declare class PathedEventEmitter implements IPathedEventEmitter {
    private listeners;
    emit(event: string | PathEvent, ...args: any[]): void;
    off(listener: Listener): void;
    on(event: string | string[], listener: Listener): Unsubscribe;
    once(event: string | string[], listener?: Listener): Promise<any>;
    relayEvents(emitter: IPathedEventEmitter): void;
}
//# sourceMappingURL=pathed-events.d.ts.map